@@ -34,16 +34,67 @@
 
 
 namespace eprosima {
-
-namespace fastrtps {
-namespace rtps {
-class EDP;
-}
-}
-
 namespace fastdds {
 namespace dds {
 
+/**
+ * The identifier for each QosPolicy.
+ *
+ * Each QosPolicy class has a different ID that is then used to refer
+ * to the incompatible policies on OfferedIncompatibleQosStatus
+ * and RequestedIncompatibleQosStatus.
+ */
+enum QosPolicyId_t : uint32_t
+{
+    INVALID_QOS_POLICY_ID                   = 0,    //< Does not refer to any valid QosPolicy
+
+    // Standard QosPolicies
+    USERDATA_QOS_POLICY_ID                  = 1,    //< UserDataQosPolicy
+    DURABILITY_QOS_POLICY_ID                = 2,    //< DurabilityQosPolicy
+    PRESENTATION_QOS_POLICY_ID              = 3,    //< PresentationQosPolicy
+    DEADLINE_QOS_POLICY_ID                  = 4,    //< DeadlineQosPolicy
+    LATENCYBUDGET_QOS_POLICY_ID             = 5,    //< LatencyBudgetQosPolicy
+    OWNERSHIP_QOS_POLICY_ID                 = 6,    //< OwnershipQosPolicy
+    OWNERSHIPSTRENGTH_QOS_POLICY_ID         = 7,    //< OwnershipStrengthQosPolicy
+    LIVELINESS_QOS_POLICY_ID                = 8,    //< LivelinessQosPolicy
+    TIMEBASEDFILTER_QOS_POLICY_ID           = 9,    //< TimeBasedFilterQosPolicy
+    PARTITION_QOS_POLICY_ID                 = 10,   //< PartitionQosPolicy
+    RELIABILITY_QOS_POLICY_ID               = 11,   //< ReliabilityQosPolicy
+    DESTINATIONORDER_QOS_POLICY_ID          = 12,   //< DestinationOrderQosPolicy
+    HISTORY_QOS_POLICY_ID                   = 13,   //< HistoryQosPolicy
+    RESOURCELIMITS_QOS_POLICY_ID            = 14,   //< ResourceLimitsQosPolicy
+    ENTITYFACTORY_QOS_POLICY_ID             = 15,   //< EntityFactoryQosPolicy
+    WRITERDATALIFECYCLE_QOS_POLICY_ID       = 16,   //< WriterDataLifecycleQosPolicy
+    READERDATALIFECYCLE_QOS_POLICY_ID       = 17,   //< ReaderDataLifecycleQosPolicy
+    TOPICDATA_QOS_POLICY_ID                 = 18,   //< TopicDataQosPolicy
+    GROUPDATA_QOS_POLICY_ID                 = 19,   //< GroupDataQosPolicy
+    TRANSPORTPRIORITY_QOS_POLICY_ID         = 20,   //< TransportPriorityQosPolicy
+    LIFESPAN_QOS_POLICY_ID                  = 21,   //< LifespanQosPolicy
+    DURABILITYSERVICE_QOS_POLICY_ID         = 22,   //< DurabilityServiceQosPolicy
+
+    //XTypes extensions
+    DATAREPRESENTATION_QOS_POLICY_ID            = 23,   //< DataRepresentationQosPolicy
+    TYPECONSISTENCYENFORCEMENT_QOS_POLICY_ID    = 24,   //< TypeConsistencyEnforcementQosPolicy
+
+    //eProsima Extensions
+    DISABLEPOSITIVEACKS_QOS_POLICY_ID       = 25,   //< DisablePositiveACKsQosPolicy
+    PARTICIPANTRESOURCELIMITS_QOS_POLICY_ID = 26,   //< ParticipantResourceLimitsQos
+    PROPERTYPOLICY_QOS_POLICY_ID            = 27,   //< PropertyPolicyQos
+    PUBLISHMODE_QOS_POLICY_ID               = 28,   //< PublishModeQosPolicy
+    READERRESOURCELIMITS_QOS_POLICY_ID      = 29,   //< Reader ResourceLimitsQos
+    RTPSENDPOINT_QOS_POLICY_ID              = 30,   //< RTPSEndpointQos
+    RTPSRELIABLEREADER_QOS_POLICY_ID        = 31,   //< RTPSReliableReaderQos
+    RTPSRELIABLEWRITER_QOS_POLICY_ID        = 32,   //< RTPSReliableWriterQos
+    TRANSPORTCONFIG_QOS_POLICY_ID           = 33,   //< TransportConfigQos
+    TYPECONSISTENCY_QOS_POLICY_ID           = 34,   //< TipeConsistencyQos
+    WIREPROTOCOLCONFIG_QOS_POLICY_ID        = 35,   //< WireProtocolConfigQos
+    WRITERRESOURCELIMITS_QOS_POLICY_ID      = 36,   //< WriterResourceLimitsQos
+
+    NEXT_QOS_POLICY_ID                              //< Keep always the last element. For internal use only
+};
+
+using PolicyMask = std::bitset<NEXT_QOS_POLICY_ID>;
+
 /**
  * Class QosPolicy, base for all QoS policies defined for Writers and Readers.
  */
@@ -898,62 +949,63 @@ public:
  * By default, zero-sized sequence.
  * @note Mutable Qos Policy
  */
-#define TEMPLATE_DATA_QOS_POLICY(TClassName, TPid)                                     \
+// *INDENT-OFF*  (uncrustify seems to have problems with this macro)
+#define TEMPLATE_DATA_QOS_POLICY(TClassName, TPid)                                         \
     class TClassName : public GenericDataQosPolicy                                         \
     {                                                                                      \
-public:                                                                                \
-                                                                                       \
+    public:                                                                                \
+                                                                                           \
         RTPS_DllAPI TClassName()                                                           \
             : GenericDataQosPolicy(TPid)                                                   \
         {                                                                                  \
         }                                                                                  \
-                                                                                       \
+                                                                                           \
         RTPS_DllAPI TClassName(                                                            \
-            uint16_t in_length)                                                        \
+                uint16_t in_length)                                                        \
             : GenericDataQosPolicy(TPid, in_length)                                        \
         {                                                                                  \
         }                                                                                  \
-                                                                                       \
-        /** \
-         * Construct from another TClassName. \
-         * \
-         * The resulting TClassName will have the same size limits \
-         * as the input attribute \
-         * \
-         * @param data data to copy in the newly created object \
-         */                                                                            \
+                                                                                           \
+        /**                                                                                \
+         * Construct from another TClassName.                                              \
+         *                                                                                 \
+         * The resulting TClassName will have the same size limits                         \
+         * as the input attribute                                                          \
+         *                                                                                 \
+         * @param data data to copy in the newly created object                            \
+         */                                                                                \
         RTPS_DllAPI TClassName(                                                            \
-            const TClassName &data) = default;                                         \
-                                                                                       \
-        /** \
-         * Construct from underlying collection type. \
-         * \
-         * Useful to easy integration on old APIs where a traditional container was used. \
-         * The resulting TClassName will always be unlimited in size \
-         * \
-         * @param data data to copy in the newly created object \
-         */                                                                            \
+                const TClassName &data) = default;                                         \
+                                                                                           \
+        /**                                                                                \
+         * Construct from underlying collection type.                                      \
+         *                                                                                 \
+         * Useful to easy integration on old APIs where a traditional container was used.  \
+         * The resulting TClassName will always be unlimited in size                       \
+         *                                                                                 \
+         * @param data data to copy in the newly created object                            \
+         */                                                                                \
         RTPS_DllAPI TClassName(                                                            \
-            const collection_type &data)                                               \
+                const collection_type &data)                                               \
             : GenericDataQosPolicy(TPid, data)                                             \
         {                                                                                  \
         }                                                                                  \
-                                                                                       \
+                                                                                           \
         virtual RTPS_DllAPI ~TClassName() = default;                                       \
-                                                                                       \
-        /** \
-         * Copies another TClassName. \
-         * \
-         * The resulting TClassName will have the same size limit \
-         * as the input parameter, so all data in the input will be copied. \
-         * \
-         * @param b object to be copied \
-         * @return reference to the current object. \
-         */                                                                            \
+                                                                                           \
+        /**                                                                                \
+         * Copies another TClassName.                                                      \
+         *                                                                                 \
+         * The resulting TClassName will have the same size limit                          \
+         * as the input parameter, so all data in the input will be copied.                \
+         *                                                                                 \
+         * @param b object to be copied                                                    \
+         * @return reference to the current object.                                        \
+         */                                                                                \
         TClassName& operator =(                                                            \
-            const TClassName &b) = default;                                            \
-                                                                                       \
+                const TClassName& b) = default;                                            \
     };
+// *INDENT-ON*
 
 //Variable used to generate the doxygen documentation for this QoS Policies
 #ifdef DOXYGEN_DOCUMENTATION
@@ -997,7 +1049,7 @@ class TopicDataQosPolicy : public GenericDataQosPolicy
 class GroupDataQosPolicy : public GenericDataQosPolicy
 {
 };
-#endif
+#endif  // DOXYGEN_DOCUMENTATION
 
 TEMPLATE_DATA_QOS_POLICY(UserDataQosPolicy, PID_USER_DATA)
 TEMPLATE_DATA_QOS_POLICY(TopicDataQosPolicy, PID_TOPIC_DATA)
@@ -1211,13 +1263,11 @@ public:
  */
 class PartitionQosPolicy : public Parameter_t, public QosPolicy
 {
-    friend class fastrtps::rtps::EDP;
-
 public:
 
     class const_iterator
     {
-public:
+    public:
 
         typedef const_iterator self_type;
         typedef const Partition_t value_type;
@@ -1262,18 +1312,18 @@ public:
         }
 
         bool operator ==(
-                const self_type& rhs)
+                const self_type& rhs) const
         {
             return ptr_ == rhs.ptr_;
         }
 
         bool operator !=(
-                const self_type& rhs)
+                const self_type& rhs) const
         {
             return ptr_ != rhs.ptr_;
         }
 
-protected:
+    protected:
 
         /**
          * @brief Shift the pointer to the next element
@@ -1286,7 +1336,7 @@ protected:
             value_ = Partition_t(ptr_);
         }
 
-private:
+    private:
 
         //!Pointer
         const fastrtps::rtps::octet* ptr_;
@@ -1606,6 +1656,9 @@ public:
     int32_t depth;
 };
 
+//! A special value indicating an unlimited quantity
+constexpr int32_t LENGTH_UNLIMITED = -1;
+
 /**
  * Specifies the resources that the Service can consume in order to meet the requested QoS
  * @note Immutable Qos Policy
@@ -1638,6 +1691,11 @@ public:
      * By default, 100.
      */
     int32_t allocated_samples;
+    /**
+     * @brief Represents the extra number of samples available once the max_samples have been reached in the history.
+     * This makes it possible, for example, to loan samples even with a full history. By default, 1.
+     */
+    int32_t extra_samples;
 
     /**
      * @brief Constructor
@@ -1649,6 +1707,7 @@ public:
         , max_instances(10)
         , max_samples_per_instance(400)
         , allocated_samples(100)
+        , extra_samples(1)
     {
     }
 
@@ -1696,9 +1755,9 @@ public:
         , QosPolicy(false)
         , history_kind(KEEP_LAST_HISTORY_QOS)
         , history_depth(1)
-        , max_samples(-1)
-        , max_instances(-1)
-        , max_samples_per_instance(-1)
+        , max_samples(LENGTH_UNLIMITED)
+        , max_instances(LENGTH_UNLIMITED)
+        , max_samples_per_instance(LENGTH_UNLIMITED)
     {
     }
 
@@ -1749,20 +1808,20 @@ public:
      * Specifies the maximum number of data-samples the DataWriter (or DataReader) can manage across all the instances
      * associated with it. Represents the maximum samples the middleware can store for any one DataWriter (or DataReader).
      * It is inconsistent for this value to be less than max_samples_per_instance. <br>
-     * By default, -1 (Length Unlimited).
+     * By default, LENGTH_UNLIMITED.
      */
     int32_t max_samples;
     /**
      * @brief Control the ResourceLimitsQos of the implied DataReader that stores the data within the durability service.
      * Represents the maximum number of instances DataWriter (or DataReader) can manage. <br>
-     * By default, -1 (Length Unlimited).
+     * By default, LENGTH_UNLIMITED.
      */
     int32_t max_instances;
     /**
      * @brief Control the ResourceLimitsQos of the implied DataReader that stores the data within the durability service.
      * Represents the maximum number of samples of any one instance a DataWriter(or DataReader) can manage.
      * It is inconsistent for this value to be greater than max_samples. <br>
-     * By default, -1 (Length Unlimited).
+     * By default, LENGTH_UNLIMITED.
      */
     int32_t max_samples_per_instance;
 };
@@ -2609,7 +2668,7 @@ public:
     }
 
     //!User defined transports to use alongside or in place of builtins.
-    std::vector<std::shared_ptr<fastdds::rtps::TransportDescriptorInterface> > user_transports;
+    std::vector<std::shared_ptr<fastdds::rtps::TransportDescriptorInterface>> user_transports;
 
     //!Set as false to disable the default UDPv4 implementation. <br> By default, true.
     bool use_builtin_transports;
@@ -2697,6 +2756,299 @@ public:
     fastrtps::ResourceLimitedContainerConfig matched_subscriber_allocation;
 };
 
+/**
+ * Data sharing configuration kinds
+ */
+enum DataSharingKind : fastrtps::rtps::octet
+{
+    /**
+     * Automatic configuration.
+     * DataSharing will be used if requirements are met.
+     */
+    AUTO = 0x01,
+    /**
+     * Activate the use of DataSharing.
+     * Entity creation will fail if requirements for DataSharing are not met
+     */
+    ON = 0x02,
+    /**
+     * Disable the use of DataSharing
+     */
+    OFF = 0x03
+};
+
+
+/**
+ * Qos Policy to configure the data sharing
+ * @note Immutable Qos Policy
+ */
+class DataSharingQosPolicy : public Parameter_t, public QosPolicy
+{
+public:
+
+    /**
+     * @brief Constructor
+     */
+    RTPS_DllAPI DataSharingQosPolicy()
+        : Parameter_t(PID_DATASHARING, 0)
+        , QosPolicy(true)
+    {
+        //Needed to generate the automatic domain ID
+        automatic();
+    }
+
+    /**
+     * @brief Destructor
+     */
+    virtual RTPS_DllAPI ~DataSharingQosPolicy() = default;
+
+    /**
+     * @brief Copy constructor
+     * @param b Another DataSharingQosPolicy instance
+     */
+    RTPS_DllAPI DataSharingQosPolicy(
+            const DataSharingQosPolicy& b)
+        : Parameter_t(b)
+        , QosPolicy(b)
+        , kind_(b.kind())
+        , shm_directory_ (b.shm_directory())
+        , max_domains_ (b.max_domains())
+        , domain_ids_(b.max_domains() != 0 ?
+                b.max_domains() :
+                b.domain_ids().size())
+    {
+        domain_ids_ = b.domain_ids();
+    }
+
+    RTPS_DllAPI DataSharingQosPolicy& operator =(
+            const DataSharingQosPolicy& b)
+    {
+        QosPolicy::operator =(b);
+        Parameter_t::operator =(b);
+        kind_ = b.kind();
+        shm_directory_ = b.shm_directory();
+        max_domains_ = b.max_domains();
+        domain_ids_.reserve(max_domains_ != 0 ?
+                max_domains_ :
+                b.domain_ids().size());
+        domain_ids_ = b.domain_ids();
+
+        return *this;
+    }
+
+    bool operator ==(
+            const DataSharingQosPolicy& b) const
+    {
+        return kind_ == b.kind_ &&
+               shm_directory_ == b.shm_directory_ &&
+               domain_ids_ == b.domain_ids_ &&
+               Parameter_t::operator ==(b) &&
+               QosPolicy::operator ==(b);
+    }
+
+    inline void clear() override
+    {
+        DataSharingQosPolicy reset = DataSharingQosPolicy();
+        std::swap(*this, reset);
+    }
+
+    /**
+     * @return the current DataSharing configuration mode
+     */
+    RTPS_DllAPI const DataSharingKind& kind() const
+    {
+        return kind_;
+    }
+
+    /**
+     * @return the current DataSharing shared memory directory
+     */
+    RTPS_DllAPI const std::string& shm_directory() const
+    {
+        return shm_directory_;
+    }
+
+    /**
+     * Gets the set of DataSharing domain IDs.
+     *
+     * Each domain ID is 64 bit long.
+     * However, user-defined domain IDs are only 16 bit long,
+     * while the rest of the 48 bits are used for the
+     * automatically generated domain ID (if any).
+     *
+     * - Automatic domain IDs use the 48 MSB and leave the 16 LSB as zero.
+     * - User defined domain IDs use the 16 LSB and leave the 48 MSB as zero.
+     *
+     * @return the current DataSharing domain IDs
+     */
+    RTPS_DllAPI const std::vector<uint64_t>& domain_ids() const
+    {
+        return domain_ids_;
+    }
+
+    /**
+     * @param size the new maximum number of domain IDs
+     */
+    RTPS_DllAPI void set_max_domains(
+            uint32_t size)
+    {
+        domain_ids_.reserve(size);
+        max_domains_ = size;
+    }
+
+    /**
+     * @return the current configured maximum number of domain IDs
+     */
+    RTPS_DllAPI const uint32_t& max_domains() const
+    {
+        return max_domains_;
+    }
+
+    /**
+     * @brief Configures the DataSharing in automatic mode
+     *
+     * The default shared memory directory of the OS is used.
+     * A default domain ID is automatically computed.
+     */
+    RTPS_DllAPI void automatic()
+    {
+        setup (AUTO, "", std::vector<uint16_t>());
+    }
+
+    /**
+     * @brief Configures the DataSharing in automatic mode
+     *
+     * The default shared memory directory of the OS is used.
+     *
+     * @param domain_ids the user configured DataSharing domain IDs (16 bits).
+     */
+    RTPS_DllAPI void automatic(
+            const std::vector<uint16_t>& domain_ids)
+    {
+        setup (AUTO, "", domain_ids);
+    }
+
+    /**
+     * @brief Configures the DataSharing in automatic mode
+     *
+     * A default domain ID is automatically computed.
+     *
+     * @param directory The shared memory directory to use.
+     */
+    RTPS_DllAPI void automatic(
+            const std::string& directory)
+    {
+        setup (AUTO, directory, std::vector<uint16_t>());
+    }
+
+    /**
+     * @brief Configures the DataSharing in automatic mode
+     *
+     * @param directory The shared memory directory to use.
+     * @param domain_ids the user configured DataSharing domain IDs (16 bits).
+     */
+    RTPS_DllAPI void automatic(
+            const std::string& directory,
+            const std::vector<uint16_t>& domain_ids)
+    {
+        setup (AUTO, directory, domain_ids);
+    }
+
+    /**
+     * @brief Configures the DataSharing in active mode
+     *
+     * A default domain ID is automatically computed.
+     *
+     * @param directory The shared memory directory to use.
+     *      It is mandatory to provide a non-empty name or the creation of endpoints will fail.
+     */
+    RTPS_DllAPI void on(
+            const std::string& directory)
+    {
+        // TODO [ILG]: This parameter is unused right now. Activate the assert once it is used
+        //assert(!directory.empty());
+        setup (ON, directory, std::vector<uint16_t>());
+    }
+
+    /**
+     * @brief Configures the DataSharing in active mode
+     *
+     * @param directory The shared memory directory to use.
+     *      It is mandatory to provide a non-empty name or the creation of endpoints will fail.
+     * @param domain_ids the user configured DataSharing domain IDs (16 bits).
+     */
+    RTPS_DllAPI void on(
+            const std::string& directory,
+            const std::vector<uint16_t>& domain_ids)
+    {
+        // TODO [ILG]: This parameter is unused right now. Activate the assert once it is used
+        //assert(!directory.empty());
+        setup (ON, directory, domain_ids);
+    }
+
+    /**
+     * @brief Configures the DataSharing in disabled mode
+     */
+    RTPS_DllAPI void off()
+    {
+        setup (OFF, "", std::vector<uint16_t>());
+    }
+
+    /**
+     * @brief Adds a user-specific DataSharing domain ID
+     *
+     * @param id 16 bit identifier
+     */
+    RTPS_DllAPI void add_domain_id(
+            uint16_t id)
+    {
+        if (max_domains_ == 0 || domain_ids_.size() < max_domains_)
+        {
+            domain_ids_.push_back(id);
+        }
+    }
+
+    // Not on the exported API, but must be public for other internal classes
+    void add_domain_id(
+            uint64_t id)
+    {
+        if (max_domains_ == 0 || domain_ids_.size() < max_domains_)
+        {
+            domain_ids_.push_back(id);
+        }
+    }
+
+private:
+
+    void setup(
+            const DataSharingKind& kind,
+            const std::string& directory,
+            const std::vector<uint16_t>& domain_ids)
+    {
+        kind_ = kind;
+        shm_directory_ = directory;
+        domain_ids_.clear();
+
+        for (uint16_t id : domain_ids)
+        {
+            add_domain_id(id);
+        }
+    }
+
+    //! DataSharing configuration mode
+    DataSharingKind kind_ = AUTO;
+
+    //! Shared memory directory to use on DataSharing
+    std::string shm_directory_;
+
+    //! Maximum number of domain IDs
+    uint32_t max_domains_ = 0;
+
+    //! Only endpoints with matching domain IDs are DataSharing compatible
+    std::vector<uint64_t> domain_ids_;
+};
+
+
 } // namespace dds
 } // namespace fastdds
 } // namespace eprosima
