@@ -13,16 +13,17 @@
 // limitations under the License.
 
 /*!
- * @file fixed_size_string.hpp
+ * @file fixed_size_bitmap.hpp
  *
  */
 
 #ifndef FASTRTPS_UTILS_FIXED_SIZE_BITMAP_HPP_
 #define FASTRTPS_UTILS_FIXED_SIZE_BITMAP_HPP_
 
-#include <cstdint>
 #include <array>
+#include <cstdint>
 #include <string.h>
+#include <limits>
 
 #if _MSC_VER
 #include <intrin.h>
@@ -32,6 +33,8 @@
 namespace eprosima {
 namespace fastrtps {
 
+using std::uint32_t;
+
 template <class T>
 struct DiffFunction
 {
@@ -42,6 +45,7 @@ struct DiffFunction
     {
         return a - b;
     }
+
 };
 
 /**
@@ -114,7 +118,7 @@ public:
         base_ = base;
         range_max_ = base_ + (NBITS - 1);
         num_bits_ = 0;
-        bitmap_.fill(0UL);
+        bitmap_.fill(0u);
     }
 
     /**
@@ -228,8 +232,8 @@ public:
             if (diff < num_bits_)
             {
                 uint32_t pos = diff >> 5;
-                diff &= 31UL;
-                return (bitmap_[pos] & (1UL << (31UL - diff))) != 0;
+                diff &= 31u;
+                return (bitmap_[pos] & (1u << (31u - diff))) != 0;
             }
         }
 
@@ -276,7 +280,7 @@ public:
             const T& from,
             const T& to)
     {
-        constexpr uint32_t full_mask = 0xFFFFFFFF;
+        constexpr uint32_t full_mask = std::numeric_limits<uint32_t>::max();
 
         // Adapt incoming range to range limits
         T min = (base_ >= from) ? base_ : from;
@@ -308,7 +312,7 @@ public:
             pos++;                              // Go to next position in the array
             n_bits -= bits_in_mask;             // Decrease number of pending bits
             mask = full_mask;                   // Mask with all bits set
-            bits_in_mask = 32UL;                // All bits set in mask (32)
+            bits_in_mask = 32u;                 // All bits set in mask (32)
         }
 
         // This condition will be true if the last bits of the mask should not be used
@@ -368,7 +372,7 @@ public:
      * This method is designed to be used when performing deserialization of a bitmap range.
      *
      * @param num_bits   Number of significant bits in the input bitmap.
-     * @param bitmap     Points to the begining of a uint32_t array holding the input bitmap.
+     * @param bitmap     Points to the beginning of a uint32_t array holding the input bitmap.
      */
     void bitmap_set(
             uint32_t num_bits,
@@ -379,6 +383,10 @@ public:
         uint32_t num_bytes = num_items * static_cast<uint32_t>(sizeof(uint32_t));
         bitmap_.fill(0u);
         memcpy(bitmap_.data(), bitmap, num_bytes);
+        if (0 < num_bits)
+        {
+            bitmap_[num_items - 1] &= ~(std::numeric_limits<uint32_t>::max() >> (num_bits & 31u));
+        }
         calc_maximum_bit_set(num_items, 0);
     }
 
@@ -442,7 +450,7 @@ private:
         {
             // Shifting more than most significant. Clear whole bitmap.
             num_bits_ = 0;
-            bitmap_.fill(0UL);
+            bitmap_.fill(0u);
         }
         else
         {
@@ -451,7 +459,7 @@ private:
 
             // Div and mod by 32
             uint32_t n_items = n_bits >> 5;
-            n_bits &= 31UL;
+            n_bits &= 31u;
             if (n_bits == 0)
             {
                 // Shifting a multiple of 32 bits, just move the bitmap integers
@@ -487,7 +495,7 @@ private:
         {
             // Shifting more than total bitmap size. Clear whole bitmap.
             num_bits_ = 0;
-            bitmap_.fill(0UL);
+            bitmap_.fill(0u);
         }
         else
         {
@@ -498,7 +506,7 @@ private:
 
             // Div and mod by 32
             uint32_t n_items = n_bits >> 5;
-            n_bits &= 31UL;
+            n_bits &= 31u;
             if (n_bits == 0)
             {
                 // Shifting a multiple of 32 bits, just move the bitmap integers
